# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:

"""
Classes that track game state.

Note that these classes track game state, but do not implement game rules.  The
only validations are to prevent changes that literally cannot be represented in
game state, such as selecting an invalid square.  All other rules (such as the
restriction that only one pawn can occupy a space, or the act of sliding down a
slider, etc.) are implemented elsewhere, using the methods available on these
classes.

Attributes:
    MIN_PLAYERS(int): Minimum number of players in a game
    MAX_PLAYERS(int): Maximum number of players in a game
    RED(str): The red player color
    BLUE(str): The blue player color
    YELLOW(str): The yellow player color
    GREEN(str): The green player color
    COLORS(list): All available player colors, listed in order of use
    PAWNS(int): Number of pawns per player
    SAFE_SQUARES(int): Number of safe squares for each color
    BOARD_SQUARES(int): Number of squares around the outside of the board
"""

from collections import OrderedDict

# A game consists of 2-4 players
MIN_PLAYERS = 2
MAX_PLAYERS = 4

# Each player gets one color
RED = "RED"
BLUE = "BLUE"
YELLOW = "YELLOW"
GREEN = "GREEN"
COLORS = [RED, YELLOW, GREEN, BLUE]  # order chosen so game works better for < 4 players

# There are 4 pawns per player, numbered 0-3
PAWNS = 4

# There are 5 safe squares for each color, numbered 0-4
SAFE_SQUARES = 5

# There are 60 squares around the outside of the board, numbered 0-59
BOARD_SQUARES = 60


class Pawn:
    """
    A pawn on the board, belonging to a player.

    Attributes:
        color(str): The color of this pawn
        index(int): Zero-based index of this pawn for a given user
        name(str): The full name of this pawn as "color-index"
        start(boolean): Whether this pawn resides in its start area
        home(boolean): Whether this pawn resides in its home area
        safe(int): Zero-based index of the square in the safe area where this pawn resides
        square(int): Zero-based index of the square on the board where this pawn resides
    """

    def __init__(self, color, index):
        """
        Create a pawn.

        Args:
            color(str): The color of this pawn
            index(int): Zero-based index of this pawn for a given user
        """
        self.color = color
        self.index = index
        self.name = "%s-%s" % (color, index)
        self.start = True
        self.home = False
        self.safe = None
        self.square = None

    def __repr__(self):
        return "Pawn(name=%s, start=%s, home=%s, safe=%s, square=%s)" % (
            self.name,
            self.start,
            self.home,
            self.safe,
            self.square,
        )

    def move_to_start(self):
        """
        Move to the pawn to its start area.
        """

        self.start = True
        self.home = False
        self.safe = None
        self.square = None

    def move_to_home(self):
        """
        Move to the pawn to its home area.
        """

        self.start = False
        self.home = True
        self.safe = None
        self.square = None

    def move_to_safe(self, square):
        """
        Move to the pawn to a square in its safe area.

        Args:
            square(int): Zero-based index of the square in the safe area

        Raises:
            ValueError: If the square is not valid
        """

        if not square in range(SAFE_SQUARES):
            raise ValueError("Invalid square")
        self.start = False
        self.home = False
        self.safe = square
        self.square = None

    def move_to_square(self, square):
        """
        Move to the pawn to a square on the board.

        Args:
            square(int): Zero-based index of the square on the board where this pawn resides

        Raises:
            ValueError: If the square is not valid
        """

        if not square in range(BOARD_SQUARES):
            raise ValueError("Invalid square")
        self.start = False
        self.home = False
        self.safe = None
        self.square = square


class Pawns(list):
    """
    A list of all pawns belonging to a player.

    Attributes:
        color(str): The color of the pawns
    """

    def __init__(self, color):
        """
        Create a complete set of pawns.

        Args:
            color(str): The color of the pawns
        """
        super(Pawns, self).__init__()
        self.color = color
        for index in range(0, PAWNS):
            self.append(Pawn(color, index))


class Player:
    """
    A player, which has a color and a set of pawns.

    Attributes:
        color(str): The color of the player
        name(str, optional): The name of the player
        pawns(Pawns): List of all pawns belonging to the player
    """

    def __init__(self, color, name=None):
        """
        Create a player.

        Args:
            color(str): The color of the player
            name(str, optional): The name of the player
        """
        self.color = color
        self.name = name
        self.pawns = Pawns(color)

    def __repr__(self):
        return "Player(%s, %s): %s" % (self.color, self.name, self.pawns)


class Players(OrderedDict):
    """
    A dict containing all players in the game.
    """

    def __init__(self, players):
        """
        Create players, allocated out of COLORS in order.

        Args:
            players(int): The number of players in the game
        """
        super(Players, self).__init__()
        if players < MIN_PLAYERS or players > MAX_PLAYERS:
            raise ValueError("Invalid number of players")
        for color in COLORS[:players]:
            self[color] = Player(color)

    def __iter__(self):
        return (player for player in self.values())


class Game:
    """
    The game, consisting of state for a set of players.

    Attributes:
        players(Players): A dict containing all players in the game.
    """

    def __init__(self, players=MAX_PLAYERS):
        """
        Create a new game.

        Args:
            players(int, optional): The number of players in the game
        """
        self.players = Players(players)

    def __str__(self):
        return "Game(players=%s)" % self.players



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# pylint: disable=line-too-long

"""
Render a game on the terminal, for development purposes.
"""

# This is kind of ugly.  It was built by hand and all of the mappings, etc. are
# hardcoded.  Since it's intended as a development and debugging tool, I don't
# really care too much about how nice it looks or how generalizable it is.

# Note: indexes into the rendered board output aren't easily predictible,
# because of the variable length of terminal formatting characters

from .game import RED, BLUE, YELLOW, GREEN

# ANSI terminal escapes for colors
# See also: http://www.lihaoyi.com/post/BuildyourownCommandLinewithANSIescapecodes.html#256-colors
_FG_BLACK = "\u001b[30;1m"
_FG_RED = "\u001b[31;1m"
_FG_GREEN = "\u001b[32;1m"
_FG_YELLOW = "\u001b[33;1m"
_FG_BLUE = "\u001b[34;1m"
_FG_MAGENTA = "\u001b[35;1m"
_FG_CYAN = "\u001b[36;1m"
_FG_WHITE = "\u001b[37;1m"
_BG_BLACK = "\u001b[40m"
_BG_RED = "\u001b[41m"
_BG_GREEN = "\u001b[42m"
_BG_YELLOW = "\u001b[43m"
_BG_BLUE = "\u001b[44m"
_BG_MAGENTA = "\u001b[45m"
_BG_CYAN = "\u001b[46m"
_BG_WHITE = "\u001b[47m"
_RESET_COLORS = "\u001b[0m"

# Index into _BOARD_TEXT where a pawn can be placed into a specific square
# Squares are numbered from starting from the upper left, in a clockwise direction
# Only a single pawn can occupy a square
# fmt: off
_SQUARE = [ 
    323, 340, 345, 350, 355, 372, 377, 382, 387, 404, 409, 414, 419, 424, 441, 446, 
    980, 1534, 2088, 2625, 3145, 3631, 4117, 4603, 5089, 5575, 6112, 6649, 7203, 7774, 8288, 8271, 
    8266, 8261, 8256, 8239, 8234, 8229, 8224, 8207, 8202, 8197, 8192, 8187, 8170, 8165, 7631, 7077, 
    6523, 5986, 5466, 4980, 4494, 4008, 3522, 3036, 2499, 1962, 1408, 837,
]
# fmt: on

# Player names as displayed on the board
_PLAYER = {}
_PLAYER[RED] = "r"
_PLAYER[BLUE] = "b"
_PLAYER[YELLOW] = "y"
_PLAYER[GREEN] = "g"

# Indexes in _BOARD_TEXT where a pawn can be placed into a start location, for each player
# There are 4 arbitrary spaces for each of the 4 available pawns
_START = {}
_START[RED] = [1281, 1283, 1285, 1287]
_START[BLUE] = [2416, 2418, 2420, 2422]
_START[YELLOW] = [7145, 7147, 7149, 7151]
_START[GREEN] = [6010, 6012, 6014, 6016]

# Indexes in _BOARD_TEXT where a pawn can be placed into a safe location, for each player
# There are 5 safe squares per color; only a single pawn can occupy a safe square
_SAFE = {}
_SAFE[RED] = [864, 1435, 1989, 2526, 3063]
_SAFE[BLUE] = [1529, 1524, 1519, 1514, 1509]
_SAFE[YELLOW] = [7747, 7176, 6622, 6085, 5548]
_SAFE[GREEN] = [7082, 7087, 7092, 7097, 7102]

# Indexes in _BOARD_TEXT where a pawn can be placed into a home location, for each player
# There are 4 arbitrary home spaces for the 4 available pawns per player
_HOME = {}
_HOME[RED] = [3708, 3710, 3712, 3714]
_HOME[BLUE] = [1318, 1320, 1322, 1324]
_HOME[YELLOW] = [4735, 4737, 4739, 4741]
_HOME[GREEN] = [7108, 7110, 7112, 7114]

# Hardcoded representation of the board, with {} placeholders for color formatting (applied below)
# The board layout was built by hand and then the formatting was placed into it by hand
_BOARD_TEXT = """{}{}{}
{}{}      0    1    2    3    4    5    6    7    8    9    10   11   12   13   14   15{}
{}{}    {}{}┌───┐{}{}┌───┐┌───┐┌───┐┌───┐{}{}┌───┐┌───┐┌───┐┌───┐{}{}┌───┐┌───┐┌───┐┌───┐┌───┐{}{}┌───┐┌───┐{}{}   {}
{}{}    {}{}│   │{}{}| ▶ || ◼ || ◼ || ● |{}{}|   ||   ||   ||   |{}{}| ▶ || ◼ || ◼ || ◼ || ● |{}{}|   ||   |{}{}   {}
{}{}    {}{}└───┘{}{}└───┘└───┘└───┘└───┘{}{}└───┘└───┘└───┘└───┘{}{}└───┘└───┘└───┘└───┘└───┘{}{}└───┘└───┘{}{}   {}
{}{}    {}{}┌───┐{}     {}{}┌───┐{}                                                            {}{}┌───┐{}{}   {}
{}{} 59 {}{}│   │{}   {}0 {}│   │{}  {}{}┌───────────┐{}       {}{}┌───────────┐{}                         {}{}| ▼ |{}{} 16{}
{}{}    {}{}└───┘{}     {}{}└───┘{}  {}{}│ S T A R T │{}       {}{}│  H O M E  │{}  4    3    2    1    0  {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}     {}{}┌───┐{}  {}{}│           │{}       {}{}│           │┌───┐┌───┐┌───┐┌───┐┌───┐┌───┐{}{}   {}
{}{} 58 {}{}│ ● │{}   1 {}{}│   │{}  {}{}│  - - - -  │{}       {}{}│  - - - -  │|   ||   ||   ||   ||   || ◼ |{}{} 17{}
{}{}    {}{}└───┘{}     {}{}└───┘{}  {}{}│  0 1 2 3  │{}       {}{}│  0 1 2 3  │└───┘└───┘└───┘└───┘└───┘└───┘{}{}   {}
{}{}    {}{}┌───┐{}     {}{}┌───┐{}  {}{}└───────────┘{}       {}{}└───────────┘{}                         {}{}┌───┐{}{}   {}
{}{} 57 {}{}│ ◼ │{}   2 {}{}│   │{}                                              {}{}┌───────────┐{} {}{}| ◼ |{}{} 18{}
{}{}    {}{}└───┘{}     {}{}└───┘{}                                              {}{}│ S T A R T │{} {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}     {}{}┌───┐{}                                              {}{}│           │{} {}{}┌───┐{}{}   {}
{}{} 56 {}{}│ ◼ │{}   3 {}{}│   │{}                                              {}{}│  - - - -  │{} {}{}| ● |{}{} 19{}
{}{}    {}{}└───┘{}     {}{}└───┘{}                                              {}{}│  0 1 2 3  │{} {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}     {}{}┌───┐{}                                              {}{}└───────────┘{} {}{}┌───┐{}{}   {}
{}{} 55 {}{}│ ◼ │{}   4 {}{}│   │{}                                                            {}{}|   |{}{} 19{}
{}{}    {}{}└───┘{}     {}{}└───┘{}                                                            {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{} {}{}┌───────────┐{}                                                        {}{}┌───┐{}{}   {}
{}{} 54 {}{}│ ▲ │{} {}{}│  H O M E  │{}                                                        {}{}|   |{}{} 20{}
{}{}    {}{}└───┘{} {}{}│           │{}                                                        {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{} {}{}│  - - - -  │{}                                                        {}{}┌───┐{}{}   {}
{}{} 53 {}{}│   │{} {}{}│  0 1 2 3  │{}                                                        {}{}|   |{}{} 21{}
{}{}    {}{}└───┘{} {}{}└───────────┘{}                                                        {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}                                                        {}{}┌───────────┐{} {}{}┌───┐{}{}   {}
{}{} 52 {}{}│   │{}                                                        {}{}│  H O M E  │{} {}{}|   |{}{} 23{}
{}{}    {}{}└───┘{}                                                        {}{}│           │{} {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}                                                        {}{}│  - - - -  │{} {}{}┌───┐{}{}   {}
{}{} 51 {}{}│   │{}                                                        {}{}│  0 1 2 3  │{} {}{}| ▼ |{}{} 24{}
{}{}    {}{}└───┘{}                                                        {}{}└───────────┘{} {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}                                                            {}{}┌───┐{}     {}{}┌───┐{}{}   {}
{}{} 50 {}{}│   │{}                                                            {}{}│   │{} 4   {}{}| ◼ |{}{} 25{}
{}{}    {}{}└───┘{} {}{}┌───────────┐{}                                              {}{}└───┘{}     {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{} {}{}│ S T A R T │{}                                              {}{}┌───┐{}     {}{}┌───┐{}{}   {}
{}{} 49 {}{}│ ● │{} {}{}│           │{}                                              {}{}│   │{} 3   {}{}| ◼ |{}{} 26{}
{}{}    {}{}└───┘{} {}{}│  - - - -  │{}                                              {}{}└───┘{}     {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{} {}{}│  0 1 2 3  │{}                                              {}{}┌───┐{}     {}{}┌───┐{}{}   {}
{}{} 48 {}{}│ ◼ │{} {}{}└───────────┘{}                                              {}{}│   │{} 2   {}{}| ◼ |{}{} 27{}
{}{}    {}{}└───┘{}                         {}{}┌───────────┐{}       {}{}┌───────────┐{}  {}{}└───┘{}     {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐┌───┐┌───┐┌───┐┌───┐┌───┐│  H O M E  │{}       {}{}│ S T A R T │{}  {}{}┌───┐{}     {}{}┌───┐{}{}   {}
{}{} 47 {}{}│ ◼ │|   ||   ||   ||   ||   |│           │{}       {}{}│           │{}  {}{}│   │{} 1   {}{}| ● |{}{} 28{}
{}{}    {}{}└───┘└───┘└───┘└───┘└───┘└───┘│  - - - -  │{}       {}{}│  - - - -  │{}  {}{}└───┘{}     {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐{}  0    1    2    3    4 {}{} │  0 1 2 3  │{}       {}{}│  0 1 2 3  │{}  {}{}┌───┐{}     {}{}┌───┐{}{}   {}
{}{} 46 {}{}│ ▲ │{}                         {}{}└───────────┘{}       {}{}└───────────┘{}  {}{}│   │{} 0   {}{}|   |{}{} 29{}
{}{}    {}{}└───┘{}                                                            {}{}└───┘{}     {}{}└───┘{}{}   {}
{}{}    {}{}┌───┐┌───┐{}{}┌───┐┌───┐┌───┐┌───┐┌───┐{}{}┌───┐┌───┐┌───┐┌───┐{}{}┌───┐┌───┐┌───┐┌───┐{}{}┌───┐{}{}   {}
{}{}    {}{}│   │|   |{}{}| ● || ◼ || ◼ || ◼ || ◀ |{}{}|   ||   ||   ||   |{}{}| ● || ◼ || ◼ || ◀ |{}{}|   |{}{}   {}
{}{}    {}{}└───┘└───┘{}{}└───┘└───┘└───┘└───┘└───┘{}{}└───┘└───┘└───┘└───┘{}{}└───┘└───┘└───┘└───┘{}{}└───┘{}{}   {}
{}{}      45   44   43   42   41   40   39   38   37   36   35   34   33   32   31   30{}
{}{}{}"""

# Color formatting, one entry for each {} placeholder in _BOARD_TEXT
# fmt: off
_BOARD_COLORS = [
    _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_RED, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _FG_WHITE, _BG_RED, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_RED, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_BLUE, _FG_WHITE, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_GREEN, _FG_WHITE, _BG_CYAN, _BG_YELLOW, _FG_WHITE, _BG_CYAN, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_YELLOW, _FG_WHITE, _BG_WHITE, _FG_BLACK, _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _RESET_COLORS,
    _BG_BLACK, _FG_WHITE, _RESET_COLORS,
]
# fmt: on


def _generate_empty_board():
    """
    Generate an empty board.
    """
    return "%s\n" % _BOARD_TEXT.format(*_BOARD_COLORS)


def _apply_game_state(board, game):
    """
    Apply game state to a board.
    """
    for player in game.players:
        for pawn in player.pawns:
            if pawn.start:
                index = _START[pawn.color][pawn.index]
            elif pawn.home:
                index = _HOME[pawn.color][pawn.index]
            elif pawn.safe is not None:
                index = _SAFE[pawn.color][pawn.safe]
            elif pawn.square is not None:
                index = _SQUARE[pawn.square]
            else:
                raise ValueError("Pawn is not in a valid state")
            board = board[:index] + _PLAYER[pawn.color] + board[index + 1 :]
    return board


def render_board(game):
    """
    Render the state of a game, returning the board for display.
    
    Args:
        game(Game): The game to render

    Returns:
        str: The board as a string for display on a terminal, including ANSI color escape sequences
    """
    return _apply_game_state(_generate_empty_board(), game)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['apologies']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'apologies',
    'version': '0.1.0a0',
    'description': 'Python code to play a game similar to Sorry',
    'long_description': "# Apologies Python Library\n\n![](https://github.com/pronovic/apologies/workflows/Test%20Suite/badge.svg)\n\nThis is a Python library that implements a game similar to the Sorry\nboard game.  It includes a rudimentary way to play the game, intended\nfor use by developers and not by end users.\n\nIt also serves as a complete example of how to manage a modern (circa 2020)\nPython project, including style checks, code formatting, integration with\nIntelliJ, continuous integration at GitHub, etc.\n\n## Developer Notes\n\n### Development Environment\n\nMy primary development environment is IntelliJ (or just Vim) on MacOS.  Notes\nbelow assume that environment, although most of this should work the same on\nWindows or Linux.\n\n### Packaging and Dependencies\n\nThis project uses [Poetry](https://python-poetry.org/) to manage Python\npackaging and dependencies.  Most day-to-day tasks (such as running unit \ntests from the command line) are orchestrated through Poetry.  A coding\nstandard is enforced using [Black](https://github.com/psf/black) and [PyLint](https://www.pylint.org/).\n\n### Developer Prequisites\n\nBefore starting, install the following tools using [Homebrew](https://brew.sh/)\nor the package manager for your platform:\n\n```shell\nbrew install python3\nbrew install poetry\nbrew install black\nbrew install pylint\n```\n\nYou need to install all of these tools before you can do local development or\ncommit code using the standard process, due to the pre-commit hooks (see\nbelow).\n\nOptionally, you may also install the following:\n\n```shell\nbrew install pre-commit   # to adjust pre-commit hooks\nbrew install make         # if you want to build Sphinx documentation\n```\n\n### Pre-Commit Hooks\n\nThere are local pre-commit hooks that depend on Black and Pylint, so the code\nis properly-formatted and lint-clean when it's checked in.  If you don't\ninstall Black and Pylint as described above, then you won't be able to commit\nyour changes.\n\nIf necessary, you can temporarily [disable a hook](https://pre-commit.com/#temporarily-disabling-hooks)\nor even remove the hook with `pre-commit uninstall`.\n\n### Activating the Virtual Environment\n\nPoetry manages the virtual environment used for testing.  Theoretically, the\nPoetry `shell` command gives you a shell using that virutalenv.  However, it\ndoesn't work that well.  Instead, it's simpler to just activate the virtual\nenvironment directly.  The [`run`](run) script has an entry that dumps out the\ncorrect `source` command. Otherwise, see [`notes/venv.sh`](notes/venv.sh) for a way\nto set up a global alias that activates any virtualenv found in the current\ndirectory.\n\n### Developer Tasks\n\nThe [`run`](run) script provides shortcuts for common developer tasks:\n\n```\n$ run --help\n\n------------------------------------\nShortcuts for common developer tasks\n------------------------------------\n\nUsage: run <command>\n\n- run install: Setup the virtualenv via Poetry\n- run activate: Print command needed to activate the Poetry virtualenv\n- run lint: Run the Pylint code checker\n- run format: Run the Black code formatter\n- run test: Run the unit tests\n- run test -c: Run the unit tests with coverage\n- run test -ch: Run the unit tests with coverage and open the HTML report\n- run tox: Run the broader Tox test suite used by the GitHub CI action\n- run docs: Build the Spinx documentation for apologies.readthedocs.io\n- run publish: Tag the current code and publish to PyPI\n```\n",
    'author': 'Kenneth J. Pronovici',
    'author_email': 'pronovic@ieee.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pronovic/apologies',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
